////////////////////////////////////////////////////////////////////////////////
// MAIN.C - Sokoban - Original from scratch source by Athlor (athlor@gmail.com)
////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "icon.h"
#include "levels.h"  // Levels
#include "Sokoban.h" // Sokoban font

char VersionPtr[] = "Sokoban";
int AppStackSize = 3000;
char map[341],flr[341],l; // Map, Floor & level
int p; // Player pos

void loadmap(void)
{
  int i; char c;
  for(i=0;i<80;i++) { // Simple transition
    LcdDrawBox(DRAW_WHITE,i,i,159-i,159-i);
    RimSleep(1);
  }
  for(i=0;i<340;i++) {
    c=levels[l][i];
    map[i]=c;
    flr[i]=(c=='!'?c:' '); // Floors (button or plain)
    if(c=='&') p=i;        // Get player pos
  }
}

void SokoThread(void)
{
  char m,h,v,e=0; // Moving, hor, vert, buttons
  char b[64]; // Buffer for sprintf
  int i,d; // Contents of direction you want to move
  MESSAGE MSG;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);
  RimRequestForeground(RimGetCurrentTaskID());
  LcdReplaceFont(0,&Sokoban);
  LcdSetCurrentFont(0);
  l=0;
  loadmap();

  for (;;) // Start message loop
  {
    LcdClearDisplay();
    for(v=0;v<17;v++) // 20x17
      for(h=0;h<20;h++)
        LcdPutStringXY(h*8,v*8,&map[v*20+h],1,0);
    LcdDrawBox(DRAW_BLACK,0,137,159,159);
    RimSprintf(b,64,"Sokoban\nLevel: %d",l+1);
    LcdSetCurrentFont(2);
    LcdPutStringXY(-1,139,b,-1,LCD_CENTERED|LCD_MULTILINE);
    LcdSetCurrentFont(0);

    RimGetMessage(&MSG);
    switch (MSG.Device)
    {
      case DEVICE_KEYPAD:
        switch (MSG.Event)
        {
          case KEY_DOWN: case KEY_REPEAT:
          {
            m=0;
            switch (MSG.SubMsg)
            {
              case KEY_ESCAPE: // ESC key exits
                RibbonShowRibbon();
                RimTerminateThread(); break;
              case 'q': d=p-20; m=1;break; // UP
              case 'z': d=p+20; m=1;break; // DOWN
              case 'a': d=p-1;  m=1;break; // LEFT
              case 's': d=p+1;  m=1;break; // RIGHT
              case 'o': if(l>0)  l--; loadmap(); break;
              case 'p': if(l<35) l++; loadmap(); break;
            }
          } // !=33  "=34  #=35  $=36
          if(m) {          // Moving
            map[p]=flr[p]; // Erase player (replace w/floor)
            if(map[d]==' '||map[d]=='!') p=d;                         // Space
            if(map[d]=='$'&&(map[d+(d-p)]==' '||map[d+(d-p)]=='!')) { // Box
              map[d+(d-p)]='$'; p=d; e=0; // Move box, player & reset button
              for(i=0;i<340;i++)
                if(map[i]=='!') e++; // Button count '!'
              if(e==0&&flr[p]!='!') {// No button check
                AlertConfiguration VIB = { // Vibration & snd structure
                 50,                 // Duty
                 50,                 // Volume (initial)
                 50,                 // Volume (max)
                 1,                  // Repetitions
                 3,                  // VibrateTime (1/10 sec)
                 CONTINUOUS_VIBRATE, // VibrateType
                 VIBRATE_AND_TONE,   // InHolsterNotify
                 VIBRATE_AND_TONE,   // OutOfHolsterNotify
                 0};                 // Pause
                map[p]='&'; // Show last piece
                RimTestAlert(TONE_SEQUENCE_2,&VIB); // Play vib & snd sequence
                if(l<35) l++;
                loadmap();
              } // End no button check
            }
            map[p]='&';    // Draw player
          } // End moving
        } // End switch(MSG.Event)
    } // End switch(MSG.Device)
  } // Message loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr,&IconBMP,0,40,SokoThread,AppStackSize);
}
